require "import"
import "android.content.Context"
import "android.os.Vibrator"
import "android.media.ToneGenerator"
import "android.media.AudioManager"
import "android.widget.*"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "android.text.Html"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(40)

local translationResult = ""
local languageCodes = {
    ["Romanian"] = "ro",
    ["English"] = "en",
    ["Albanian"] = "sq",
    ["French"] = "fr",
    ["German"] = "de",
    ["Greek"] = "el",
    ["Hungarian"] = "hu",
    ["Italian"] = "it",
    ["Portuguese (Portugal)"] = "pt-PT",
    ["Spanish"] = "es",
    ["Chinese (Simplified)"] = "zh-CN",
}

local items = {}
for language, code in pairs(languageCodes) do
    table.insert(items, language)
end
table.sort(items)

function capitalizeFirstWord(str)
    return str:gsub("^%l", string.upper)
end

function addPunctuation(str)
    return str .. "."
end

function hasInternetConnection()
    local connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

if not hasInternetConnection() then
    service.asyncSpeak("Eroare traducere, nu există conexiune la internet")
    return false
end

local layout = {
    LinearLayout;
    layout_width = "fill";
    layout_height = "fill";
    orientation = "vertical";
    {
        EditText,
        id = "searchBox";
        layout_width = "fill";
        layout_height = "wrap";
        hint = "Căutare limbă pentru traducere.";
    };
    {
        GridView,
        id = "grid";
        numColumns = 1;
        layout_width = "fill";
        layout_height = "0dp";
        layout_weight = 1;
    },
    {
        Button,
        id = "exitButton";
        text = "Ieșire";
        layout_width = "fill";
        layout_height = "wrap";
    }
}

local dlg = LuaDialog(context)
dlg.View = loadlayout(layout)

local adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, items)
grid.adapter = adapter

dlg.setTitle("Google Translate")
    .setMessage("Bun venit la Google translate!")
    .show()

searchBox.addTextChangedListener{
    onTextChanged = function(s, start, before, count)
        local keyword = s.toString()
        local filteredItems = {}
        for _, language in ipairs(items) do
            if language:lower():find(keyword:lower()) then
                table.insert(filteredItems, language)
            end
        end
        adapter.clear()
        adapter.addAll(filteredItems)
        adapter.notifyDataSetChanged()
    end
}
grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(500, function()
        local language = v.text
        local defaultLanguage = Locale.getDefault().getLanguage()
        local languageCode = languageCodes[language]

        if languageCode then
            local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(context.getText(node))
            url = url .. "&tl=" .. languageCode

            Http.get(url, function(status, result)
                if status == 200 then
                    local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
                    translation = capitalizeFirstWord(translation)
                    translation = addPunctuation(translation)
                    translationResult = translationResult .. translation
                    service.setText(translationResult)
                    service.speak("textul tradus: , " .. translationResult)
                    vibrator.vibrate(100)
                else
                    print("A apărut o eroare.\n Verificați următoarele:\n - asigurați-vă că sunteți conectat la internet.\n Notă: anumite lemente (fotografii, video etc.) nu pot fi traduse.")
                    vibrator.vibrate(200)
                    service.playSoundTick()
                end
            end)
        else
            print("Limbă nesuportată.")
            vibrator.vibrate(200)
            service.playSoundClock()
        end
    end)
end

exitButton.onClick = function()
    service.playSoundTick()
    service.speak("")
    dlg.dismiss()
end
